@@ -18,6 +18,7 @@
 #include "usb.h"
 #include "pnp.h"
 #include "loader.h"
+#include "ntoskernel_exports.h"
 
 /* MDLs describe a range of virtual address with an array of physical
  * pages right after the header. For different ranges of virtual
@@ -67,8 +68,6 @@ static void ntos_work_worker(worker_param_t dummy);
 static struct nt_thread *ntos_worker_thread;
 spinlock_t irp_cancel_lock;
 static NT_SPIN_LOCK nt_list_lock;
-
-extern struct nt_list wrap_drivers;
 static struct nt_slist wrap_timer_slist;
 
 /* compute ticks (100ns) since 1601 until when system booted into
@@ -85,9 +84,7 @@ WIN_SYMBOL_MAP("KeTickCount", &jiffies)
 
 WIN_SYMBOL_MAP("NlsMbCodePageTag", FALSE)
 
-#ifdef NTOS_WQ
 workqueue_struct_t *ntos_wq;
-#endif
 
 #ifdef WRAP_PREEMPT
 DEFINE_PER_CPU(irql_info_t, irql_info);
@@ -422,12 +419,12 @@ static void timer_proc(unsigned long data)
 	BUG_ON(wrap_timer->wrap_timer_magic != WRAP_TIMER_MAGIC);
 	BUG_ON(nt_timer->wrap_timer_magic != WRAP_TIMER_MAGIC);
 #endif
-	kdpc = nt_timer->kdpc;
-	if (kdpc)
-		queue_kdpc(kdpc);
 	KeSetEvent((struct nt_event *)nt_timer, 0, FALSE);
 	if (wrap_timer->repeat)
 		mod_timer(&wrap_timer->timer, jiffies + wrap_timer->repeat);
+	kdpc = nt_timer->kdpc;
+	if (kdpc)
+		queue_kdpc(kdpc);
 	TIMEREXIT(return);
 }
 
@@ -564,11 +561,10 @@ wstdcall BOOLEAN WIN_FUNC(KeCancelTimer,1)
 #ifdef TIMER_DEBUG
 	BUG_ON(wrap_timer->wrap_timer_magic != WRAP_TIMER_MAGIC);
 #endif
-	TIMERTRACE("canceling timer %p(%p)", wrap_timer, nt_timer);
 	/* disable timer before deleting so if it is periodic timer, it
 	 * won't be re-armed after deleting */
 	wrap_timer->repeat = 0;
-	ret = del_timer(&wrap_timer->timer);
+	ret = del_timer_sync(&wrap_timer->timer);
 	/* the documentation for KeCancelTimer suggests the DPC is
 	 * deqeued, but actually DPC is left to run */
 	if (ret)
@@ -1129,8 +1125,8 @@ wstdcall NTSTATUS WIN_FUNC(KeWaitForMultipleObjects,8)
 	DBG_BLOCK(2) {
 		KIRQL irql = current_irql();
 		if (irql >= DISPATCH_LEVEL) {
-			TRACE2("wait in atomic context: %Lu, %lu, %d, %ld",
-			       *timeout, wait_hz, in_atomic(), in_interrupt());
+			TRACE2("wait in atomic context: %lu, %d, %ld",
+			       wait_hz, in_atomic(), in_interrupt());
 		}
 	}
 	assert_irql(_irql_ < DISPATCH_LEVEL);
@@ -1445,7 +1441,7 @@ struct nt_thread *get_current_nt_thread(void)
 	return thread;
 }
 
-struct task_struct *get_nt_thread_task(struct nt_thread *thread)
+static struct task_struct *get_nt_thread_task(struct nt_thread *thread)
 {
 	struct task_struct *task;
 	struct common_object_header *header;
@@ -1485,6 +1481,7 @@ static struct nt_thread *create_nt_thread(struct task_struct *task)
 	InitializeListHead(&thread->irps);
 	initialize_object(&thread->dh, ThreadObject, 0);
 	thread->dh.size = sizeof(*thread);
+	thread->prio = LOW_PRIORITY;
 	return thread;
 }
 
@@ -1519,12 +1516,7 @@ wstdcall KPRIORITY WIN_FUNC(KeQueryPriorityThread,1)
 		EXIT2(return LOW_REALTIME_PRIORITY);
 	}
 
-	if (thread_priority(thread->task) <= 0)
-		prio = LOW_PRIORITY;
-	else if (thread_priority(thread->task) <= -5)
-		prio = LOW_REALTIME_PRIORITY;
-	else
-		prio = MAXIMUM_PRIORITY;
+	prio = thread->prio;
 
 	TRACE2("%d", prio);
 	return prio;
@@ -1551,21 +1543,10 @@ wstdcall KPRIORITY WIN_FUNC(KeSetPriorityThread,2)
 		EXIT2(return LOW_REALTIME_PRIORITY);
 	}
 
-	if (thread_priority(task) <= 0)
-		old_prio = LOW_PRIORITY;
-	else if (thread_priority(task) <= -5)
-		old_prio = LOW_REALTIME_PRIORITY;
-	else
-		old_prio = MAXIMUM_PRIORITY;
+	old_prio = thread->prio;
+	thread->prio = prio;
 
-	if (prio == LOW_PRIORITY)
-		set_thread_priority(task, 0);
-	else if (prio == LOW_REALTIME_PRIORITY)
-		set_thread_priority(task, -5);
-	else if (prio == HIGH_PRIORITY)
-		set_thread_priority(task, -10);
-
-	TRACE2("%d, %d", old_prio, (int)thread_priority(task));
+	TRACE2("%d, %d", old_prio, thread->prio);
 	return old_prio;
 }
 
@@ -1619,16 +1600,6 @@ wstdcall NTSTATUS WIN_FUNC(PsCreateSystemThread,7)
 	thread_tramp.ctx = ctx;
 	init_completion(&thread_tramp.started);
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,7)
-	thread_tramp.thread->pid = kernel_thread(ntdriver_thread, &thread_tramp,
-						 CLONE_SIGHAND);
-	TRACE2("pid = %d", thread_tramp.thread->pid);
-	if (thread_tramp.thread->pid < 0) {
-		free_object(thread_tramp.thread);
-		EXIT2(return STATUS_FAILURE);
-	}
-	TRACE2("created task: %d", thread_tramp.thread->pid);
-#else
 	thread_tramp.thread->task = kthread_run(ntdriver_thread,
 						&thread_tramp, "ntdriver");
 	if (IS_ERR(thread_tramp.thread->task)) {
@@ -1636,7 +1607,6 @@ wstdcall NTSTATUS WIN_FUNC(PsCreateSystemThread,7)
 		EXIT2(return STATUS_FAILURE);
 	}
 	TRACE2("created task: %p", thread_tramp.thread->task);
-#endif
 
 	wait_for_completion(&thread_tramp.started);
 	*handle = OBJECT_TO_HEADER(thread_tramp.thread);
@@ -1715,7 +1685,7 @@ wstdcall void *WIN_FUNC(MmAllocateContiguousMemorySpecifyCache,5)
 	 PHYSICAL_ADDRESS boundary, enum memory_caching_type cache_type)
 {
 	void *addr;
-	unsigned int flags;
+	gfp_t flags;
 
 	ENTER2("%lu, 0x%lx, 0x%lx, 0x%lx, %d", size, (long)lowest,
 	       (long)highest, (long)boundary, cache_type);
@@ -1764,11 +1734,11 @@ wstdcall PHYSICAL_ADDRESS WIN_FUNC(MmGetPhysicalAddress,1)
  * probably get this device to work if we create a buffer with the
  * strings as required by the driver and return virtual address for
  * that address instead */
-wstdcall void *WIN_FUNC(MmMapIoSpace,3)
+wstdcall void __iomem *WIN_FUNC(MmMapIoSpace,3)
 	(PHYSICAL_ADDRESS phys_addr, SIZE_T size,
 	 enum memory_caching_type cache)
 {
-	void *virt;
+	void __iomem *virt;
 	ENTER1("cache type: %d", cache);
 	if (cache == MmCached)
 		virt = ioremap(phys_addr, size);
@@ -1779,7 +1749,7 @@ wstdcall void *WIN_FUNC(MmMapIoSpace,3)
 }
 
 wstdcall void WIN_FUNC(MmUnmapIoSpace,2)
-	(void *addr, SIZE_T size)
+	(void __iomem *addr, SIZE_T size)
 {
 	ENTER1("%p, %lu", addr, size);
 	iounmap(addr);
@@ -1985,7 +1955,7 @@ wfastcall LONG WIN_FUNC(ObfReferenceObject,1)
 	return ret;
 }
 
-int dereference_object(void *object)
+static int dereference_object(void *object)
 {
 	struct common_object_header *hdr;
 	int ref_count;
@@ -2424,6 +2394,13 @@ noregparm ULONG WIN_FUNC(DbgPrint,12)
 	return STATUS_SUCCESS;
 }
 
+wstdcall void WIN_FUNC(KeBugCheck,1)
+	(ULONG code)
+{
+	TODO();
+	return;
+}
+
 wstdcall void WIN_FUNC(KeBugCheckEx,5)
 	(ULONG code, ULONG_PTR param1, ULONG_PTR param2,
 	 ULONG_PTR param3, ULONG_PTR param4)
@@ -2464,20 +2441,18 @@ wstdcall void WIN_FUNC(__C_specific_handler,0)
 	TODO();
 }
 
-void WIN_FUNC(_purecall,0)
+wstdcall void WIN_FUNC(_purecall,0)
 	(void)
 {
 	TODO();
 }
 
-void WIN_FUNC(__chkstk,0)
+wstdcall void WIN_FUNC(__chkstk,0)
 	(void)
 {
 	TODO();
 }
 
-#include "ntoskernel_exports.h"
-
 struct worker_init_struct {
 	work_struct_t work;
 	struct completion completion;
@@ -2557,20 +2532,16 @@ int ntoskernel_init(void)
 	} while (0);
 #endif
 
-#ifdef NTOS_WQ
 	ntos_wq = create_singlethread_workqueue("ntos_wq");
 	if (!ntos_wq) {
 		WARNING("couldn't create ntos_wq thread");
 		return -ENOMEM;
 	}
 	ntos_worker_thread = wrap_worker_init(ntos_wq);
-#else
-	ntos_worker_thread = wrap_worker_init(NULL);
-#endif
 	TRACE1("%p", ntos_worker_thread);
 
 	if (add_bus_driver("PCI")
-#ifdef CONFIG_USB
+#ifdef ENABLE_USB
 	    || add_bus_driver("USB")
 #endif
 		) {
@@ -2688,10 +2659,8 @@ void ntoskernel_exit(void)
 #if defined(CONFIG_X86_64)
 	del_timer_sync(&shared_data_timer);
 #endif
-#ifdef NTOS_WQ
 	if (ntos_wq)
 		destroy_workqueue(ntos_wq);
-#endif
 	TRACE1("%p", ntos_worker_thread);
 	if (ntos_worker_thread)
 		ObDereferenceObject(ntos_worker_thread);
