@@ -36,9 +36,6 @@ struct pe_exports {
 
 static struct pe_exports pe_exports[40];
 static int num_pe_exports;
-#if defined(CONFIG_X86_64)
-extern struct kuser_shared_data kuser_shared_data;
-#endif
 
 #define RVA2VA(image, rva, type) (type)(ULONG_PTR)((void *)image + rva)
 #define CHECK_SZ(a,b) { if (sizeof(a) != b) {				\
@@ -72,7 +69,7 @@ static const char *image_directory_name[] = {
 #ifndef TEST_LOADER
 extern struct wrap_export ntoskernel_exports[], ntoskernel_io_exports[],
 	ndis_exports[], crt_exports[], hal_exports[], rtl_exports[];
-#ifdef CONFIG_USB
+#ifdef ENABLE_USB
 extern struct wrap_export usb_exports[];
 #endif
 
@@ -104,7 +101,7 @@ static char *get_export(char *name)
 		if (strcmp(rtl_exports[i].name, name) == 0)
 			return (char *)rtl_exports[i].func;
 
-#ifdef CONFIG_USB
+#ifdef ENABLE_USB
 	for (i = 0 ; usb_exports[i].name != NULL; i++)
 		if (strcmp(usb_exports[i].name, name) == 0)
 			return (char *)usb_exports[i].func;
@@ -210,10 +207,10 @@ static int check_nt_hdr(IMAGE_NT_HEADERS *nt_hdr)
 			  opt_hdr->DataDirectory[i].Size);
 	}
 
-	if ((nt_hdr->FileHeader.Characteristics & IMAGE_FILE_DLL))
-		return IMAGE_FILE_DLL;
 	if ((nt_hdr->FileHeader.Characteristics & IMAGE_FILE_EXECUTABLE_IMAGE))
 		return IMAGE_FILE_EXECUTABLE_IMAGE;
+	if ((nt_hdr->FileHeader.Characteristics & IMAGE_FILE_DLL))
+		return IMAGE_FILE_DLL;
 	return -EINVAL;
 }
 
@@ -497,7 +494,7 @@ static int fix_pe_image(struct pe_image *pe)
 }
 
 #if defined(CONFIG_X86_64)
-void fix_user_shared_data_addr(char *driver, unsigned long length)
+static void fix_user_shared_data_addr(char *driver, unsigned long length)
 {
 	unsigned long i, n, max_addr, *addr;
 
@@ -508,6 +505,7 @@ void fix_user_shared_data_addr(char *driver, unsigned long length)
 		if (*addr >= KI_USER_SHARED_DATA && *addr < max_addr) {
 			*addr -= KI_USER_SHARED_DATA;
 			*addr += (unsigned long)&kuser_shared_data;
+			kuser_shared_data.reserved1 = 1;
 		}
 	}
 }
@@ -537,7 +535,7 @@ int link_pe_images(struct pe_image *pe_image, unsigned short n)
 		dos_hdr = pe->image;
 
 		if (pe->size < sizeof(IMAGE_DOS_HEADER)) {
-			DBGTRACE1("image too small: %d", pe->size);
+			TRACE1("image too small: %d", pe->size);
 			return -EINVAL;
 		}
 
@@ -547,17 +545,17 @@ int link_pe_images(struct pe_image *pe_image, unsigned short n)
 
 		pe->type = check_nt_hdr(pe->nt_hdr);
 		if (pe->type <= 0) {
-			DBGTRACE1("type <= 0");
+			TRACE1("type <= 0");
 			return -EINVAL;
 		}
 
 		if (fix_pe_image(pe)) {
-			DBGTRACE1("bad PE image");
+			TRACE1("bad PE image");
 			return -EINVAL;
 		}
 
 		if (read_exports(pe)) {
-			DBGTRACE1("read exports failed");
+			TRACE1("read exports failed");
 			return -EINVAL;
 		}
 	}
@@ -566,11 +564,11 @@ int link_pe_images(struct pe_image *pe_image, unsigned short n)
 	        pe = &pe_image[i];
 
 		if (fixup_reloc(pe->image, pe->nt_hdr)) {
-			DBGTRACE1("fixup reloc failed");
+			TRACE1("fixup reloc failed");
 			return -EINVAL;
 		}
 		if (fixup_imports(pe->image, pe->nt_hdr)) {
-			DBGTRACE1("fixup imports failed");
+			TRACE1("fixup imports failed");
 			return -EINVAL;
 		}
 #if defined(CONFIG_X86_64)
@@ -582,8 +580,8 @@ int link_pe_images(struct pe_image *pe_image, unsigned short n)
 		pe->entry =
 			RVA2VA(pe->image,
 			       pe->opt_hdr->AddressOfEntryPoint, void *);
-		DBGTRACE1("entry is at %p, rva at %08X", pe->entry,
-			  pe->opt_hdr->AddressOfEntryPoint);
+		TRACE1("entry is at %p, rva at %08X", pe->entry,
+		       pe->opt_hdr->AddressOfEntryPoint);
 	}
 
 	for (i = 0; i < n; i++) {
@@ -599,7 +597,7 @@ int link_pe_images(struct pe_image *pe_image, unsigned short n)
 			ustring.buf = (wchar_t *)buf;
 			dll_entry = (void *)get_dll_init(pe->name);
 
-			DBGTRACE1("calling dll_init at %p", dll_entry);
+			TRACE1("calling dll_init at %p", dll_entry);
 			if (!dll_entry || dll_entry(&ustring))
 				ERROR("DLL initialize failed for %s",
 				      pe->name);
