@@ -16,6 +16,7 @@
 
 #include "ndis.h"
 #include "usb.h"
+#include "usb_exports.h"
 
 #ifdef USB_DEBUG
 static unsigned int urb_id = 0;
@@ -55,11 +56,7 @@ static unsigned int urb_id = 0;
 
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,5)
 #define CUR_ALT_SETTING(intf) (intf)->cur_altsetting
-#else
-#define CUR_ALT_SETTING(intf) (intf)->altsetting[(intf)->act_altsetting]
-#endif
 
 #ifndef USB_CTRL_SET_TIMEOUT
 #define USB_CTRL_SET_TIMEOUT 5000
@@ -93,41 +90,10 @@ static int inline wrap_cancel_urb(struct wrap_urb *wrap_urb)
 	}
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-
-#define URB_STATUS(wrap_urb) (wrap_urb->urb_status)
-
-static void *usb_buffer_alloc(struct usb_device *udev, size_t size,
-			      unsigned mem_flags, dma_addr_t *dma)
-{
-	*dma = 0;
-	return kmalloc(size, mem_flags);
-}
-
-static void usb_buffer_free(struct usb_device *udev, size_t size, void *addr,
-			    dma_addr_t dma)
-{
-	kfree(addr);
-}
-
-static void usb_init_urb(struct urb *urb)
-{
-	memset(urb, 0, sizeof(*urb));
-}
-
-static void usb_kill_urb(struct urb *urb)
-{
-	usb_unlink_urb(urb);
-}
-
-#else
-
 #define URB_STATUS(wrap_urb) (wrap_urb->urb->status)
 
-#endif
-
 static struct nt_list wrap_urb_complete_list;
-static NT_SPIN_LOCK wrap_urb_complete_list_lock;
+static spinlock_t wrap_urb_complete_list_lock;
 
 static work_struct_t wrap_urb_complete_work;
 static void wrap_urb_complete_worker(worker_param_t dummy);
@@ -138,7 +104,7 @@ static void kill_all_urbs(struct wrap_device *wd, int complete)
 	struct wrap_urb *wrap_urb;
 	KIRQL irql;
 
-	USBENTER("%d", wd->usb.num_alloc_urbs);
+	USBTRACE("%d", wd->usb.num_alloc_urbs);
 	while (1) {
 		IoAcquireCancelSpinLock(&irql);
 		ent = RemoveHeadList(&wd->usb.wrap_urb_list);
@@ -167,7 +133,7 @@ static USBD_STATUS wrap_urb_status(int urb_status)
 	case 0:
 		return USBD_STATUS_SUCCESS;
 	case -EPROTO:
-		return USBD_STATUS_BTSTUFF;
+		return USBD_STATUS_TIMEOUT;
 	case -EILSEQ:
 		return USBD_STATUS_CRC;
 	case -EPIPE:
@@ -200,7 +166,7 @@ static NTSTATUS nt_urb_irp_status(USBD_STATUS nt_urb_status)
 	case USBD_STATUS_SUCCESS:
 		return STATUS_SUCCESS;
 	case USBD_STATUS_DEVICE_GONE:
-		return STATUS_DEVICE_NOT_CONNECTED;
+		return STATUS_DEVICE_REMOVED;
 	case USBD_STATUS_PENDING:
 		return STATUS_PENDING;
 	case USBD_STATUS_NOT_SUPPORTED:
@@ -281,13 +247,13 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 				  void *buf, unsigned int buf_len)
 {
 	struct urb *urb;
-	unsigned int alloc_flags;
+	gfp_t alloc_flags;
 	struct wrap_urb *wrap_urb;
 	struct wrap_device *wd;
 
 	USBENTER("irp: %p", irp);
 	wd = IRP_WRAP_DEVICE(irp);
-	alloc_flags = gfp_irql();
+	alloc_flags = irql_gfp();
 	IoAcquireCancelSpinLock(&irp->cancel_irql);
 	urb = NULL;
 	nt_list_for_each_entry(wrap_urb, &wd->usb.wrap_urb_list, list) {
@@ -300,22 +266,17 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 	}
 	if (!urb) {
 		IoReleaseCancelSpinLock(irp->cancel_irql);
-		wrap_urb = kmalloc(sizeof(*wrap_urb), alloc_flags);
+		wrap_urb = kzalloc(sizeof(*wrap_urb), alloc_flags);
 		if (!wrap_urb) {
 			WARNING("couldn't allocate memory");
 			return NULL;
 		}
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 		urb = usb_alloc_urb(0, alloc_flags);
-#else
-		urb = usb_alloc_urb(0);
-#endif
 		if (!urb) {
 			WARNING("couldn't allocate urb");
 			kfree(wrap_urb);
 			return NULL;
 		}
-		memset(wrap_urb, 0, sizeof(*wrap_urb));
 		IoAcquireCancelSpinLock(&irp->cancel_irql);
 		wrap_urb->urb = urb;
 		wrap_urb->state = URB_ALLOCATED;
@@ -334,7 +295,7 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 	/* called as Windows function */
 	irp->cancel_routine = WIN_FUNC_PTR(wrap_cancel_irp,2);
 	IoReleaseCancelSpinLock(irp->cancel_irql);
-	USBTRACE("allocated urb: %p", urb);
+	USBTRACE("urb: %p", urb);
 
 	urb->transfer_buffer_length = buf_len;
 	if (buf_len && buf && (!virt_addr_valid(buf)
@@ -359,8 +320,7 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 		wrap_urb->flags |= WRAP_URB_COPY_BUFFER;
 		if (usb_pipeout(pipe))
 			memcpy(urb->transfer_buffer, buf, buf_len);
-		USBTRACE("DMA buffer for urb %p is %p",
-			 urb, urb->transfer_buffer);
+		USBTRACE("DMA buf for urb %p: %p", urb, urb->transfer_buffer);
 	} else
 		urb->transfer_buffer = buf;
 	return urb;
@@ -391,11 +351,7 @@ static USBD_STATUS wrap_submit_urb(struct irp *irp)
 	DUMP_URB_BUFFER(urb, USB_DIR_OUT);
 	USBTRACE("%p", urb);
 	IRP_WRAP_URB(irp)->state = URB_SUBMITTED;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-	ret = usb_submit_urb(urb, gfp_irql());
-#else
-	ret = usb_submit_urb(urb);
-#endif
+	ret = usb_submit_urb(urb, irql_gfp());
 	if (ret) {
 		USBTRACE("ret: %d", ret);
 		wrap_free_urb(urb);
@@ -407,26 +363,7 @@ static USBD_STATUS wrap_submit_urb(struct irp *irp)
 		USBEXIT(return USBD_STATUS_PENDING);
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-static void int_urb_unlink_complete(struct urb *urb)
-{
-	struct wrap_urb *wrap_urb;
-
-	wrap_urb = urb->context;
-	USBTRACE("%p, %d, %d", urb, urb->status, wrap_urb->state);
-	if (xchg(&wrap_urb->state, URB_INT_UNLINKED) != URB_COMPLETED)
-		return;
-	urb->status = URB_STATUS(wrap_urb);
-	nt_spin_lock(&wrap_urb_complete_list_lock);
-	InsertTailList(&wrap_urb_complete_list, &wrap_urb->complete_list);
-	nt_spin_unlock(&wrap_urb_complete_list_lock);
-	schedule_ntos_work(&wrap_urb_complete_work);
-}
-
-static void wrap_urb_complete(struct urb *urb)
-#else
 static void wrap_urb_complete(struct urb *urb ISR_PT_REGS_PARAM_DECL)
-#endif
 {
 	struct irp *irp;
 	struct wrap_urb *wrap_urb;
@@ -444,22 +381,10 @@ static void wrap_urb_complete(struct urb *urb ISR_PT_REGS_PARAM_DECL)
 	}
 #endif
 	wrap_urb->state = URB_COMPLETED;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-	URB_STATUS(wrap_urb) = urb->status;
-	/* prevent 2.4 kernels from resubmitting interrupt urbs;
-	 * completion function for unlink will process urb */
-	if (usb_pipeint(urb->pipe)) {
-		USBTRACE("%p, %d", urb, urb->status);
-		urb->complete = int_urb_unlink_complete;
-		usb_unlink_urb(urb);
-		return;
-	}
-#endif
-	nt_spin_lock(&wrap_urb_complete_list_lock);
+	spin_lock(&wrap_urb_complete_list_lock);
 	InsertTailList(&wrap_urb_complete_list, &wrap_urb->complete_list);
-	nt_spin_unlock(&wrap_urb_complete_list_lock);
+	spin_unlock(&wrap_urb_complete_list_lock);
 	schedule_ntos_work(&wrap_urb_complete_work);
-	USBTRACE("scheduled worker for urb %p", urb);
 }
 
 /* one worker for all devices */
@@ -476,9 +401,9 @@ static void wrap_urb_complete_worker(worker_param_t dummy)
 
 	USBENTER("");
 	while (1) {
-		nt_spin_lock_irqsave(&wrap_urb_complete_list_lock, flags);
+		spin_lock_irqsave(&wrap_urb_complete_list_lock, flags);
 		ent = RemoveHeadList(&wrap_urb_complete_list);
-		nt_spin_unlock_irqrestore(&wrap_urb_complete_list_lock, flags);
+		spin_unlock_irqrestore(&wrap_urb_complete_list_lock, flags);
 		if (!ent)
 			break;
 		wrap_urb = container_of(ent, struct wrap_urb, complete_list);
@@ -527,13 +452,13 @@ static void wrap_urb_complete_worker(worker_param_t dummy)
 		case -ECONNRESET:
 			/* urb canceled */
 			irp->io_status.info = 0;
-			TRACE1("urb %p canceled", urb);
+			TRACE2("urb %p canceled", urb);
 			NT_URB_STATUS(nt_urb) = USBD_STATUS_SUCCESS;
 			irp->io_status.status = STATUS_CANCELLED;
 			break;
 		default:
-			TRACE1("irp: %p, urb: %p, status: %d/%d",
-			       irp, urb, urb->status, wrap_urb->state);
+			TRACE2("irp: %p, urb: %p, status: %d/%d",
+				 irp, urb, urb->status, wrap_urb->state);
 			irp->io_status.info = 0;
 			NT_URB_STATUS(nt_urb) = wrap_urb_status(urb->status);
 			irp->io_status.status =
@@ -560,7 +485,7 @@ static USBD_STATUS wrap_bulk_or_intr_trans(struct irp *irp)
 	udev = IRP_WRAP_DEVICE(irp)->usb.udev;
 	bulk_int_tx = &nt_urb->bulk_int_transfer;
 	pipe_handle = bulk_int_tx->pipe_handle;
-	USBTRACE("flags: %X, length: %u, buffer: %p, handle: %p",
+	USBTRACE("flags: 0x%x, length: %u, buffer: %p, handle: %p",
 		 bulk_int_tx->transfer_flags,
 		 bulk_int_tx->transfer_buffer_length,
 		 bulk_int_tx->transfer_buffer, pipe_handle);
@@ -689,13 +614,12 @@ static USBD_STATUS wrap_vendor_or_class_req(struct irp *irp)
 		urb->transfer_flags |= URB_SHORT_NOT_OK;
 	}
 
-	dr = kmalloc(sizeof(*dr), GFP_ATOMIC);
+	dr = kzalloc(sizeof(*dr), GFP_ATOMIC);
 	if (!dr) {
 		ERROR("couldn't allocate memory");
 		wrap_free_urb(urb);
 		return USBD_STATUS_NO_MEMORY;
 	}
-	memset(dr, 0, sizeof(*dr));
 	dr->bRequestType = req_type;
 	dr->bRequest = vc_req->request;
 	dr->wValue = cpu_to_le16(vc_req->value);
@@ -774,6 +698,91 @@ static USBD_STATUS wrap_abort_pipe(struct usb_device *udev, struct irp *irp)
 	USBEXIT(return USBD_STATUS_SUCCESS);
 }
 
+static USBD_STATUS wrap_set_clear_feature(struct usb_device *udev,
+					  struct irp *irp)
+{
+	union nt_urb *nt_urb;
+	struct urb_control_feature_request *feat_req;
+	int ret = 0;
+	__u8 request, type;
+	__u16 feature;
+
+	nt_urb = IRP_URB(irp);
+	feat_req = &nt_urb->feat_req;
+	feature = feat_req->feature_selector;
+	switch (nt_urb->header.function) {
+	case URB_FUNCTION_SET_FEATURE_TO_DEVICE:
+		request = USB_REQ_SET_FEATURE;
+		type = USB_DT_DEVICE;
+		break;
+	case URB_FUNCTION_SET_FEATURE_TO_INTERFACE:
+		request = USB_REQ_SET_FEATURE;
+		type =  USB_DT_INTERFACE;
+		break;
+	case URB_FUNCTION_SET_FEATURE_TO_ENDPOINT:
+		request = USB_REQ_SET_FEATURE;
+		type =  USB_DT_ENDPOINT;
+		break;
+	case URB_FUNCTION_CLEAR_FEATURE_TO_DEVICE:
+		request = USB_REQ_CLEAR_FEATURE;
+		type =  USB_DT_DEVICE;
+		break;
+	case URB_FUNCTION_CLEAR_FEATURE_TO_INTERFACE:
+		request = USB_REQ_CLEAR_FEATURE;
+		type =  USB_DT_INTERFACE;
+		break;
+	case URB_FUNCTION_CLEAR_FEATURE_TO_ENDPOINT:
+		request = USB_REQ_CLEAR_FEATURE;
+		type =  USB_DT_ENDPOINT;
+		break;
+	default:
+		WARNING("invalid function: %x", nt_urb->header.function);
+		NT_URB_STATUS(nt_urb) = USBD_STATUS_NOT_SUPPORTED;
+		return NT_URB_STATUS(nt_urb);
+	}
+	ret = usb_control_msg(udev, usb_sndctrlpipe(udev, 0), request, type,
+			      feature, feat_req->index, NULL, 0, 1000);
+	NT_URB_STATUS(nt_urb) = wrap_urb_status(ret);
+	USBEXIT(return NT_URB_STATUS(nt_urb));
+}
+
+static USBD_STATUS wrap_get_status_request(struct usb_device *udev,
+					   struct irp *irp)
+{
+	union nt_urb *nt_urb;
+	struct urb_control_get_status_request *status_req;
+	int ret = 0;
+	__u8 type;
+
+	nt_urb = IRP_URB(irp);
+	status_req = &nt_urb->status_req;
+	switch (nt_urb->header.function) {
+	case URB_FUNCTION_GET_STATUS_FROM_DEVICE:
+		type = USB_RECIP_DEVICE;
+		break;
+	case URB_FUNCTION_GET_STATUS_FROM_INTERFACE:
+		type = USB_RECIP_INTERFACE;
+		break;
+	case URB_FUNCTION_GET_STATUS_FROM_ENDPOINT:
+		type = USB_RECIP_ENDPOINT;
+		break;
+	default:
+		WARNING("invalid function: %x", nt_urb->header.function);
+		NT_URB_STATUS(nt_urb) = USBD_STATUS_NOT_SUPPORTED;
+		return NT_URB_STATUS(nt_urb);
+	}
+	assert(status_req->transfer_buffer_length == sizeof(u16));
+	ret = usb_get_status(udev, type, status_req->index,
+			     status_req->transfer_buffer);
+	if (ret >= 0) {
+		assert(ret <= status_req->transfer_buffer_length);
+		status_req->transfer_buffer_length = ret;
+		NT_URB_STATUS(nt_urb) = USBD_STATUS_SUCCESS;
+	} else
+		NT_URB_STATUS(nt_urb) = wrap_urb_status(ret);
+	USBEXIT(return NT_URB_STATUS(nt_urb));
+}
+
 static void set_intf_pipe_info(struct wrap_device *wd,
 			       struct usb_interface *usb_intf,
 			       struct usbd_interface_information *intf)
@@ -782,13 +791,8 @@ static void set_intf_pipe_info(struct wrap_device *wd,
 	struct usb_endpoint_descriptor *ep;
 	struct usbd_pipe_information *pipe;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	for (i = 0; i < CUR_ALT_SETTING(usb_intf)->desc.bNumEndpoints; i++) {
 		ep = &(CUR_ALT_SETTING(usb_intf)->endpoint[i]).desc;
-#else
-	for (i = 0; i < CUR_ALT_SETTING(usb_intf).bNumEndpoints; i++) {
-		ep = &((CUR_ALT_SETTING(usb_intf)).endpoint[i]);
-#endif
 		if (i >= intf->bNumEndpoints) {
 			ERROR("intf %p has only %d endpoints, "
 			      "ignoring endpoints above %d",
@@ -803,22 +807,50 @@ static void set_intf_pipe_info(struct wrap_device *wd,
 		USBTRACE("driver wants max_tx_size to %d",
 			 pipe->max_tx_size);
 
-		pipe->wMaxPacketSize = ep->wMaxPacketSize;
+		pipe->wMaxPacketSize = le16_to_cpu(ep->wMaxPacketSize);
 		pipe->bEndpointAddress = ep->bEndpointAddress;
 		pipe->type = ep->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK;
 		if (pipe->type == UsbdPipeTypeInterrupt) {
-			/* for low speed devices, Linux sets bInterval
-			 * as frames per second, whereas Windows
-			 * interprets it as milliseconds */
+			/* Windows and Linux differ in how the
+			 * bInterval is interpretted */
+			/* for low speed:
+			   interval (Windows) -> frames per ms (Linux)
+			   0 to 15    -> 8
+			   16 to 35   -> 16
+			   36 to 255  -> 32
+
+			   for full speed: interval -> frames per ms
+			   1          -> 1
+			   2 to 3     -> 2
+			   4 to 7     -> 4
+			   8 to 15    -> 8
+			   16 to 31   -> 16
+			   32 to 255  -> 32
+
+			   for high speed: interval -> microframes
+			   1          -> 1
+			   2          -> 2
+			   3          -> 4
+			   4          -> 8
+			   5          -> 16
+			   6          -> 32
+			   7 to 255   -> 32
+			*/
 			if (wd->usb.udev->speed == USB_SPEED_LOW)
-				pipe->bInterval = 1 << (ep->bInterval - 1);
-			else
+				pipe->bInterval = ep->bInterval + 5;
+			else if (wd->usb.udev->speed == USB_SPEED_FULL)
 				pipe->bInterval = ep->bInterval;
+			else {
+				int j, k;
+				for (j = k = 1; j < ep->bInterval; k++)
+					j *= 2;
+				pipe->bInterval = k;
+			}
 		}
 		pipe->handle = ep;
 		USBTRACE("%d: ep 0x%x, type %d, pkt_sz %d, intv %d (%d),"
 			 "type: %d, handle %p", i, ep->bEndpointAddress,
-			 ep->bmAttributes, ep->wMaxPacketSize, ep->bInterval,
+			 ep->bmAttributes, pipe->wMaxPacketSize, ep->bInterval,
 			 pipe->bInterval, pipe->type, pipe->handle);
 	}
 }
@@ -840,11 +872,7 @@ static USBD_STATUS wrap_select_configuration(struct wrap_device *wd,
 	USBTRACE("%p", config);
 	if (config == NULL) {
 		kill_all_urbs(wd, 1);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 		ret = usb_reset_configuration(udev);
-#else
-		ret = 0;
-#endif
 		return wrap_urb_status(ret);
 	}
 
@@ -1029,6 +1057,21 @@ static USBD_STATUS wrap_process_nt_urb(struct irp *irp)
 		status = wrap_abort_pipe(udev, irp);
 		break;
 
+	case URB_FUNCTION_SET_FEATURE_TO_DEVICE:
+	case URB_FUNCTION_SET_FEATURE_TO_INTERFACE:
+	case URB_FUNCTION_SET_FEATURE_TO_ENDPOINT:
+	case URB_FUNCTION_CLEAR_FEATURE_TO_DEVICE:
+	case URB_FUNCTION_CLEAR_FEATURE_TO_INTERFACE:
+	case URB_FUNCTION_CLEAR_FEATURE_TO_ENDPOINT:
+		status = wrap_set_clear_feature(udev, irp);
+		break;
+
+	case URB_FUNCTION_GET_STATUS_FROM_DEVICE:
+	case URB_FUNCTION_GET_STATUS_FROM_INTERFACE:
+	case URB_FUNCTION_GET_STATUS_FROM_ENDPOINT:
+		status = wrap_get_status_request(udev, irp);
+		break;
+
 	default:
 		ERROR("function %x not implemented", nt_urb->header.function);
 		status = NT_URB_STATUS(nt_urb) = USBD_STATUS_NOT_SUPPORTED;
@@ -1045,48 +1088,39 @@ static USBD_STATUS wrap_reset_port(struct irp *irp)
 
 	wd = IRP_WRAP_DEVICE(irp);
 	USBENTER("%p, %p", wd, wd->usb.udev);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10)
 	lock = usb_lock_device_for_reset(wd->usb.udev, wd->usb.intf);
 	if (lock < 0) {
 		WARNING("locking failed: %d", lock);
-		return wrap_urb_status(lock);
+//		return wrap_urb_status(lock);
+		return USBD_STATUS_SUCCESS;
 	}
-#endif
 	ret = usb_reset_device(wd->usb.udev);
 	if (ret < 0)
 		USBTRACE("reset failed: %d", ret);
 	/* TODO: should reconfigure? */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10)
 	if (lock)
 		usb_unlock_device(wd->usb.udev);
-#endif
-	return wrap_urb_status(ret);
+//	return wrap_urb_status(ret);
+	return USBD_STATUS_SUCCESS;
 }
 
 static USBD_STATUS wrap_get_port_status(struct irp *irp)
 {
 	struct wrap_device *wd;
 	ULONG *status;
+	enum usb_device_state state;
 
 	wd = IRP_WRAP_DEVICE(irp);
 	USBENTER("%p, %p", wd, wd->usb.udev);
 	status = IoGetCurrentIrpStackLocation(irp)->params.others.arg1;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-	{
-		enum usb_device_state state;
-		state = wd->usb.udev->state;
-		if (state != USB_STATE_NOTATTACHED &&
-		    state != USB_STATE_SUSPENDED) {
-			*status |= USBD_PORT_CONNECTED;
-			if (state == USB_STATE_CONFIGURED)
-				*status |= USBD_PORT_ENABLED;
-		}
-		USBTRACE("state: %d, *status: %08X", state, *status);
+	state = wd->usb.udev->state;
+	if (state != USB_STATE_NOTATTACHED &&
+	    state != USB_STATE_SUSPENDED) {
+		*status |= USBD_PORT_CONNECTED;
+		if (state == USB_STATE_CONFIGURED)
+			*status |= USBD_PORT_ENABLED;
 	}
-#else
-	/* TODO: how to get current status? */
-	*status = USBD_PORT_CONNECTED | USBD_PORT_ENABLED;
-#endif
+	USBTRACE("state: %d, *status: %08X", state, *status);
 	return USBD_STATUS_SUCCESS;
 }
 
@@ -1097,10 +1131,14 @@ NTSTATUS wrap_submit_irp(struct device_object *pdo, struct irp *irp)
 	USBD_STATUS status;
 	struct usbd_idle_callback *idle_callback;
 
-	USBTRACE("%p, %p", pdo, irp);
+	USBENTER("%p, %p", pdo, irp);
 	wd = pdo->reserved;
-//	if (wd->usb.intf == NULL)
-//		USBEXIT(return STATUS_DEVICE_REMOVED);
+	if (wd->usb.intf == NULL) {
+		USBTRACE("%p", irp);
+		irp->io_status.status = STATUS_DEVICE_REMOVED;
+		irp->io_status.info = 0;
+		USBEXIT(return STATUS_DEVICE_REMOVED);
+	}
 	IRP_WRAP_DEVICE(irp) = wd;
 	irp_sl = IoGetCurrentIrpStackLocation(irp);
 	switch (irp_sl->params.dev_ioctl.code) {
@@ -1221,7 +1259,8 @@ WIN_FUNC(USBD_ParseConfigurationDescriptorEx,7)
 		 bInterfaceNumber, bAlternateSetting, bInterfaceClass,
 		 bInterfaceSubClass, bInterfaceProtocol);
 
-	for (pos = start; pos < ((void *)config + config->wTotalLength);
+	for (pos = start;
+	     pos < ((void *)config + le16_to_cpu(config->wTotalLength));
 	     pos += intf->bLength) {
 
 		intf = pos;
@@ -1388,12 +1427,10 @@ USBD_InterfaceLogEntry(void *context, ULONG driver_tag, ULONG enum_tag,
 	USBEXIT(return STATUS_SUCCESS);
 }
 
-#include "usb_exports.h"
-
 int usb_init(void)
 {
 	InitializeListHead(&wrap_urb_complete_list);
-	nt_spin_lock_init(&wrap_urb_complete_list_lock);
+	spin_lock_init(&wrap_urb_complete_list_lock);
 	initialize_work(&wrap_urb_complete_work, wrap_urb_complete_worker, NULL);
 #ifdef USB_DEBUG
 	urb_id = 0;
